#include <iostream>
#include <string>
#include <vector>

#include "appl_grid/appl_grid.h"
#include "appl_grid/appl_timer.h"

#include "TH1D.h"
#include "TPad.h"


// lhapdf routines
#include "LHAPDF/LHAPDF.h"
extern "C" void evolvepdf_(const double& , const double& , double* ); 
extern "C" double alphaspdf_(const double& Q);



int main(int argc, char** argv) { 

  // use a default atlas inclusive grid
  std::string gridname = "atlas-incljets04-eta1.root";

  if ( argc>1 ) gridname = argv[1];

  std::cout << "reading grid " << gridname << std::endl; 

  // get name of grid from user and create from grid file
  appl::grid g(gridname);

  g.trim(); // trim away uneeded memory

  std::cout << "setting up lhapdf" << std::endl; 

  // initialise lhapdf

  //  const string _pdfname = "cteq6mE.LHgrid";  
  const string _pdfname = "CT10.LHgrid";  
  int iset = 0;

  LHAPDF::initPDFSet( _pdfname, iset );
  // initpdfset_(_pdfname.c_str());
  // initpdf_( iset );

  // do the convolution into a vector
  std::cout << "doing standalone convolution" << std::endl; 
  std::cout << g.getDocumentation() << std::endl; 

  
  struct timeval atimer = appl_timer_start(); 

  std::vector<double>  xsec = g.vconvolute( evolvepdf_, alphaspdf_ ); 

  double atime = appl_timer_stop(atimer); 

  const int Nsubproc = g.subProcesses();
  
  std::cout << "time : " << atime << " ms\tNsubproc : " << Nsubproc << std::endl;

  std::vector<std::vector<double> >  xsb(Nsubproc);
  
  for ( int i=0 ; i<Nsubproc ; i++ ) { 
    xsb[i] = g.vconvolute( i, evolvepdf_, alphaspdf_ ); 
  }


  // print results
  for ( int i=0 ; i<xsec.size() ; i++ ) { 
    std::cout << "xsec(" << i << ")=" << xsec[i] << std::endl;
  }

  for ( int i=0 ; i<Nsubproc ; i++ ) { 
    std::cout << "subproc: " << i << std::endl;
    for ( int j=0 ; j<xsec.size() ; j++ ) { 
      std::cout << "xsec(" << i << ")=" << xsb[i][j] << std::endl;
    } 
  }





  TH1D*  hxsec = g.convolute( evolvepdf_, alphaspdf_ ); 

  std::vector<TH1D*>  hxsb(Nsubproc);
  
  for ( int i=0 ; i<Nsubproc ; i++ ) { 
    hxsb[i] = g.convolute( i, evolvepdf_, alphaspdf_ ); 
  }


  hxsec->SetLineColor(kRed);
  hxsec->SetMarkerColor(kRed);
  hxsec->SetMarkerStyle(20);
  hxsec->SetMinimum(0.000001);
  hxsec->DrawCopy();
  gPad->SetLogy(true);
  for ( int i=0 ; i<Nsubproc ; i++ ) { 
    hxsb[i]->SetLineColor(kRed+i%5);
    hxsb[i]->SetMarkerColor(kRed+i%5);
    hxsb[i]->SetLineStyle(1+i%5);
    hxsb[i]->SetMarkerStyle(20+i%5);
    hxsb[i]->DrawCopy("same");
  }


  gPad->Print("xsec.pdf");


  double escale = 7e3/8e3;
  TH1D*  hxsec8 = g.convolute( escale, evolvepdf_, alphaspdf_ ); 

  escale = 7e3/2.76e3;
  TH1D*  hxsec276 = g.convolute( escale, evolvepdf_, alphaspdf_ ); 


  hxsec->DrawCopy();
  hxsec8->SetLineColor(kRed+1);
  hxsec8->SetLineStyle(2);
  hxsec8->DrawCopy("same");

  gPad->Print("xsec8.pdf");

  hxsec276->SetLineColor(kRed+2);
  hxsec276->SetLineStyle(3);
  hxsec276->DrawCopy("same");

  gPad->Print("xsec276.pdf");

  //  or get into a histogram
  //  TH1D* hxsec = g.convolute( evolvepdf_, alphaspdf_ ); 
  //  hxsec->SetName("xsec");

  return 0;
}
